/**
 * JASMINe
 * Copyright (C) 2008-2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: QuickVisuService.java 5438 2009-09-29 13:15:49Z jlegrand $
 * --------------------------------------------------------------------------
 */

package org.ow2.jasmine.monitoring.eos.quickvisu.service;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

import org.ow2.jasmine.monitoring.eos.common.JasmineEventManager;
import org.ow2.jasmine.monitoring.eos.common.jms.JMSConnection;
import org.ow2.jasmine.monitoring.eos.common.jms.event.JasmineMessageEvent;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

public class QuickVisuService implements Serializable {

    /**
     *  serialVersionUID.
     */
    private static final long serialVersionUID = -1254916345191626735L;

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(QuickVisuService.class);

    /**
     * The JMS connection instance.
     */
    private JMSConnection connection = null;

    /**
     * The event manager instance.
     */
    private JasmineEventManager manager = null;

    /**
     * A Timer to prevent from disconnection.
     */
    private Timer timer = null;

    // 2 minutes
    private static long TIMER = 120000;

    public QuickVisuService() {
        this.manager = new JasmineEventManager();
    }

    /**
     * connect to the JMS topic.
     * @return -1 if an error occured, 0 otherwise.
     */
    public int connectTopic() {
        if (this.connection == null) {
            this.connection = new JMSConnection(this.manager);
        }

        // set the timer
        this.timer = new Timer();
        this.timer.schedule(new DisconnectTask(), QuickVisuService.TIMER);

        return this.connection.connect();
    }

    /**
     * Disconnect from the JMS topic.
     * @return -1 if an error occurred, 0 otherwise.
     */
    public int disconnectTopic() {
        logger.debug("Disconnecting from the JMS topic...");
        try {

            if (this.connection == null) {
                return -1;
            }
            return this.connection.disconnect();
        } finally {
            if (this.timer != null) {
                this.timer.cancel();
            }
        }

    }

    /**
     * Get all the messages in the queue.
     * @return a list of events.
     */
    public List<JasmineMessageEvent> getMessage() {

        this.resetTimer();

        List<JasmineMessageEvent> list = new ArrayList<JasmineMessageEvent>();

        list.addAll(this.manager.getAll());

        return list;
    }

    /**
     * Reset the timer.
     */
    private void resetTimer() {
        this.timer.cancel();
        this.timer = new Timer();
        this.timer.schedule(new DisconnectTask(), QuickVisuService.TIMER);
    }

    /**
     * Implementation of TimerTask.
     * @author Guillaume Renault
     *
     */
    class DisconnectTask extends TimerTask {

        @Override
        public void run() {
            disconnectTopic();

        }

    }

}
